#ifndef FS_FILEWRITER_H
#define FS_FILEWRITER_H

#include <string>
#include <list>
#include "fs.h"

class FileWriter
{
public:
	enum Type {
									IMMEDIATE,
									SYNC,
									ASYNC,
									MT
	};
	
	typedef BOOL					(CALLBACK* SfvdProto) (HANDLE, LONGLONG);

	static void						SetMaxMemSize(int inMemSize);
	static void						SetType(Type inType);
	static FileWriter*				GetFileWriter(const std::string& inName, long long inFileSize);
	static void						SetReadCallback(BufferCallbackFunc inReadCallback)		{ mReadCallback = inReadCallback; }
	static BufferCallbackFunc		GetReadCallback()										{ return mReadCallback; }
	static void						SetWriteCallback(BufferCallbackFunc inWriteCallback)	{ mWriteCallback = inWriteCallback; }
	static BufferCallbackFunc		GetWriteCallback()										{ return mWriteCallback; }
	static void						Finalize();

	static SfvdProto				GetSetFileValidDataFunction();
	static void						UseSetFileValid(bool inUseSetFileValid)					{ mUseSetFileValid = inUseSetFileValid; }
	static void						UsePreAllocation(bool inUsePreAllocation)				{ mUsePreAllocation = inUsePreAllocation; }

	virtual							~FileWriter()											{ }

	virtual bool					Open()													= 0;
	virtual bool					Close()													= 0;

	virtual void					GetWriteBuffer(unsigned char*& outBuf, int& outLength)	= 0;
	virtual bool					ReleaseWriteBuffer(int inRemainingLength)				= 0;
	
	virtual void					Finish()												{ }
	virtual bool					IsFinished()											= 0;

	static void						AbortSync();
	bool							AbortAcknowledged()										{ return mAbortAcknowledged; }
	virtual bool					IsAborting();
	virtual void					HandleAbort()											= 0;
	
	const std::string&				GetName()												{ return mName; }

	HANDLE							CreateFile(bool inDoOverlappedIO);
	bool							CloseFile(HANDLE& ioHandle, long long inWritten = -1);
	void							DeleteFile();

protected:
									FileWriter(const std::string& inName, long long inFileSize);

	std::string						mName;
	long long						mFileSize;
	bool							mAbortAcknowledged;
	
	static int						mMaxMemSize;
	static Type						mType;
	static std::list<FileWriter*>	mActiveFileWriters;
	static BufferCallbackFunc		mReadCallback;
	static BufferCallbackFunc		mWriteCallback;
	static bool						mAbortRequested;

	static bool						mUseSetFileValid;
	static bool						mUsePreAllocation;
};

#endif